/*
 * tpdfview - src/tpdfview.c
 * Copyright (C) 2011 Sebastian 'tokkee' Harl <sh@tokkee.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * A tiny PDF viewer.
 */

#if HAVE_CONFIG_H
#	include "config.h"
#endif /* HAVE_CONFIG_H */

#include "tpdfv.h"
#include "tpdfv_features.h"

#include "gtk-tpdfv.h"
#include "tpdfv-ctl.h"

#if HAVE_LIBGEN_H
#	include <libgen.h>
#else /* HAVE_LIBGEN_H */
#	define basename(path) (path)
#endif /* ! HAVE_LIBGEN_H */

#include <assert.h>

#include <stdio.h>
#include <stdlib.h>

#include <string.h>

#include <unistd.h>

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

/*
 * Global variables.
 */
static gtk_tpdfv_screens_t *screens = NULL;

static void
exit_usage(char *name, int status)
{
	printf(
"Usage: %s [<options>] <filename>\n"

"\nA tiny PDF viewer.\n"

"\nOptions:\n"
"  -P    enable presentation mode\n"
"\n"
"  -h    display this help and exit\n"
"  -V    display the version number and copyright\n"

"\ntpdfview "TPDFV_VERSION_STRING TPDFV_VERSION_EXTRA", "PACKAGE_URL"\n"
"Copyright (C) 2011 "PACKAGE_MAINTAINER"\n",
basename(name));
	exit(status);
} /* exit_usage */

static void
exit_version(void)
{
	printf("tpdfview version "TPDFV_VERSION_STRING TPDFV_VERSION_EXTRA", "
			"built "BUILD_DATE"\n"
			"Copyright (C) 2011 "PACKAGE_MAINTAINER"\n"

			"\nThis is free software under the terms of the BSD license, see "
			"the source for\ncopying conditions. There is NO WARRANTY; not "
			"even for MERCHANTABILITY or\nFITNESS FOR A PARTICULAR "
			"PURPOSE.\n");
	exit(0);
} /* exit_version */

static void
toggle_fullscreen(GtkWindow *win)
{
	GdkWindowState state;

	if (! gtk_widget_get_realized(GTK_WIDGET(win)))
		return;

	state = gdk_window_get_state(GTK_WIDGET(win)->window);

	if (state & GDK_WINDOW_STATE_FULLSCREEN)
		gtk_window_unfullscreen(win);
	else
		gtk_window_fullscreen(win);
} /* toggle_fullscreen */

static void
on_destroy(GtkWidget __attribute__((unused)) *widget,
		gpointer __attribute__((unused)) data)
{
	gtk_main_quit();
} /* on_destroy */

static gboolean
key_press(GtkWidget *window, GdkEventKey *event, gpointer data)
{
	GObject *obj;

	obj = G_OBJECT(data);
	assert(obj);

	switch (event->keyval) {
		case GDK_q:
			gtk_main_quit();
			break;

		case GDK_r:
			tpdfv_ctl_reload(obj);
			break;

		case GDK_F:
			toggle_fullscreen(GTK_WINDOW(window));
			break;

		/* navigation */
		case GDK_Page_Up:
			tpdfv_ctl_page_up(obj);
			break;
		case GDK_Page_Down:
			/* fall through */
		case GDK_space:
			tpdfv_ctl_page_down(obj);
			break;
		case GDK_Home:
			tpdfv_ctl_first_page(obj);
			break;
		case GDK_End:
			tpdfv_ctl_last_page(obj);
			break;

		/* zoom */
		case GDK_plus:
			tpdfv_ctl_zoom_in(obj);
			break;
		case GDK_minus:
			tpdfv_ctl_zoom_out(obj);
			break;
		case GDK_1:
			tpdfv_ctl_zoom_1(obj);
			break;
		case GDK_w:
			tpdfv_ctl_zoom_width(obj);
			break;
		case GDK_h:
			tpdfv_ctl_zoom_height(obj);
			break;
		case GDK_z:
			tpdfv_ctl_zoom_fit(obj);
			break;

		/* scrolling */
		case GDK_Up:
			tpdfv_ctl_scroll_up(obj);
			break;
		case GDK_Down:
			tpdfv_ctl_scroll_down(obj);
			break;
		case GDK_Left:
			tpdfv_ctl_scroll_left(obj);
			break;
		case GDK_Right:
			tpdfv_ctl_scroll_right(obj);
			break;

		/* screen management */
		case GDK_M:
			{
				gint n_screens = gtk_tpdfv_screens_number(screens);
				gint screen = gtk_tpdfv_screens_window_get(screens,
						GTK_WINDOW(window));

				gtk_tpdfv_screens_window_set(screens,
						GTK_WINDOW(window), (screen + 1) % n_screens);
			}
			break;
	}
	return FALSE;
} /* key_press */

int
main(int argc, char **argv)
{
	GtkWidget *win;
	GtkWidget *tpdfv;

	GObject *ctl;

	char win_title[1024];

	_Bool presentation_mode = 0;

	GdkColor bg_color;

	char *filename;

	gtk_init(&argc, &argv);

	while (42) {
		int opt = getopt(argc, argv, "PhV");

		if (-1 == opt)
			break;

		switch (opt) {
			case 'P':
				presentation_mode = 1;
				break;

			case 'h':
				exit_usage(argv[0], 0);
				break;
			case 'V':
				exit_version();
				break;
			default:
				exit_usage(argv[0], 1);
		}
	}

	if (argc - optind != 1) {
		fprintf(stderr, "%s: missing filename\n", argv[0]);
		exit_usage(argv[0], 1);
	}

	filename = argv[optind];

	tpdfv = gtk_tpdfv_new(filename);
	if (! tpdfv)
		return 1;

	win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	if (! win)
		return 1;

	screens = gtk_tpdfv_screens_init(/* display = */ NULL);
	if (! screens)
		return 1;

	snprintf(win_title, sizeof(win_title), "tpdfview: %s",
			basename(filename));
	gtk_window_set_title(GTK_WINDOW(win), win_title);

	gtk_container_add(GTK_CONTAINER(win), tpdfv);

	ctl = tpdfv_ctl_new(GTK_TPDFV(tpdfv));
	if (! ctl)
		return 1;

	/* TODO: use resource file */
	gdk_color_parse("#000000", &bg_color);

	if (presentation_mode) {
		GtkWidget *pres_win;
		GtkWidget *paned;
		GtkWidget *tpdfv1, *tpdfv2;

		tpdfv1 = gtk_tpdfv_new(filename);
		if (! tpdfv1)
			return 1;

		if (! tpdfv_ctl_add_slave(ctl, GTK_TPDFV(tpdfv1),
					/* factor = */ 1, /* offset = */ 0))
			return 1;

		tpdfv2 = gtk_tpdfv_new(filename);
		if (! tpdfv2)
			return 1;

		if (! tpdfv_ctl_add_slave(ctl, GTK_TPDFV(tpdfv2),
					/* factor = */ 1, /* offset = */ 1))
			return 1;

		paned = gtk_hpaned_new();
		if (! paned)
			return 1;

		gtk_paned_pack1(GTK_PANED(paned), tpdfv1, TRUE, TRUE);
		gtk_paned_pack2(GTK_PANED(paned), tpdfv2, TRUE, TRUE);

		gtk_tpdfv_zoom_fit(tpdfv1);
		gtk_tpdfv_zoom_fit(tpdfv2);

		pres_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		if (! pres_win)
			return 1;

		gtk_window_set_title(GTK_WINDOW(pres_win), win_title);

		gtk_container_add(GTK_CONTAINER(pres_win), paned);

		gtk_widget_modify_bg(tpdfv1, GTK_STATE_NORMAL, &bg_color);
		gtk_widget_modify_bg(tpdfv2, GTK_STATE_NORMAL, &bg_color);

		g_signal_connect(G_OBJECT(pres_win), "destroy",
				G_CALLBACK(on_destroy), NULL);
		g_signal_connect(G_OBJECT(pres_win), "key-press-event",
				G_CALLBACK(key_press), ctl);

		gtk_widget_modify_bg(pres_win, GTK_STATE_NORMAL, &bg_color);
		gtk_widget_modify_bg(tpdfv1, GTK_STATE_NORMAL, &bg_color);
		gtk_widget_modify_bg(tpdfv2, GTK_STATE_NORMAL, &bg_color);

		gtk_widget_show_all(pres_win);
	}

	g_signal_connect(G_OBJECT(win), "destroy",
			G_CALLBACK(on_destroy), NULL);
	g_signal_connect(G_OBJECT(win), "key-press-event",
			G_CALLBACK(key_press), ctl);

	gtk_widget_modify_bg(tpdfv, GTK_STATE_NORMAL, &bg_color);

	gtk_widget_show_all(win);

	gtk_main();

	gtk_tpdfv_screens_destroy(screens);
	return 0;
} /* main */

/* vim: set tw=78 sw=4 ts=4 noexpandtab : */

